﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Damage")

L:RegisterTranslations("enUS", function() return {
	["Damage done"] = true,
	["D"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Damage done"] = "伤害输出",
	["D"] = "伤害(D)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Damage done"] = "造成傷害",
	["D"] = "傷害(D)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage done"] = "입힌 피해량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Damage done"] = "Dégâts infligés",
	["D"] = "D",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Damage done"
local roster = AceLibrary("Roster-2.1")
local parserEvent = {
	eventType = "Damage",
	sourceID_not = false,
}


-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "damage"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Damage done"] end
function mod:GetShortDisplayName() return L["D"] end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local amount
	if eventType == "SWING_DAMAGE" then
		amount = ...
	elseif eventType == "RANGE_DAMAGE" or eventType == "SPELL_DAMAGE" or eventType == "SPELL_PERIODIC_DAMAGE" or eventType == "DAMAGE_SHIELD" or eventType == "DAMAGE_SPLIT" then
		amount = select(4, ...)
	end
	if amount and amount > 0 then
		local isSmember = bit.band(srcFlags, LIB_FILTER_GROUP)
		isSmember = (isSmember == COMBATLOG_OBJECT_TYPE_PLAYER or isSmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isSmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		local isDmember = bit.band(dstFlags, LIB_FILTER_GROUP)
		isDmember = (isDmember == COMBATLOG_OBJECT_TYPE_PLAYER or isDmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isDmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isSmember and not isDmember then
			self:OnData(srcName, amount)
		end
	end
end

