﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HPS")

L:RegisterTranslations("enUS", function() return {
	["Healing per Second"] = true,
	["HPS"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Healing per Second"] = "每秒治疗",
	["HPS"] = "每秒治疗(HPS)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Healing per Second"] = "每秒治療",
	["HPS"] = "每秒治療(HPS)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing per Second"] = "초당 치유량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Healing per Second"] = "Soins par seconde",
	["HPS"] = "SPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local seconds = nil
local healingDone = nil
local moduleName = "Healing per Second"
local isInCombat = {}
local combatStartTime = {}
local lastHealTime = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")

local parserEvent = {
	eventType = "Heal",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.hidePercentage = true
mod.violationCategory = "healing"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
	mod:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	mod:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnReset()
	if type(seconds) == "table" then
		seconds = mod:del(seconds)
	end
	seconds = mod:new()
	
	if type(healingDone) == "table" then
		healingDone = mod:del(healingDone)
	end
	healingDone = mod:new()

	for k in pairs(combatStartTime) do
		combatStartTime[k] = nil
	end
	for k in pairs(lastHealTime) do
		lastHealTime[k] = nil
	end
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Healing per Second"] end
function mod:GetShortDisplayName() return L["HPS"] end

function mod:OnRemoteData(name, input)
	if name and input then
		self.dirty = (input ~= (self.data[name] or 0))
		self.data[name] = input
	end
end

function mod:OnData(name, input)
	if not self:IsLocal(name) and self.data[name] then
		return
	end	
	self.data[name] = nil
	self.dirty = true
end

function mod:IsDirty()
	return self.dirty or next(isInCombat)
end

function mod:IsBroadcastDirty( charName )
	if not charName then
		charName = self.playerName
	end
	return ( self.broadcast[charName] ~= self:GetData(charName) ) or isInCombat[charName]
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max((lastHealTime[name] or GetTime()) - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5)
end

local function calculateData(name)
	return math.floor((healingDone[name] or 0) / getDuration(name))
end

local moduleData = {}
function mod:GetData(name)
	if name then
		return self.data[name] or calculateData(name)
	else
		for k, v in pairs( moduleData ) do
			moduleData[k] = nil
		end
		for name in pairs(healingDone) do
			moduleData[name] = calculateData(name)
		end
		for name, value in pairs(self.data) do
			moduleData[name] = value 
		end
		return moduleData
	end
end

function mod:GetBroadcastData( charName )
	if not charName then
		charName = self.playerName
	end
	self.broadcast[charName] = self:GetData(charName)
	return self.broadcast[charName]
end

function mod:UpdateHPS(name)
	self:OnData(name, true)
end

function mod:ResetHPSTimer(name)
	if combatStartTime[name] then
		seconds[name] = getDuration(name)
		
		combatStartTime[name] = nil
		lastHealTime[name] = nil
		
		self:UpdateHPS(name)
	end
end

function mod:CheckHPSTimer(name)
	if lastHealTime[name] and lastHealTime[name] + COMBAT_TIMEOUT < GetTime() then
		self:ResetHPSTimer(name)
	end
end

local function getOverheal( unit, amount )
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			return amount - d
		end
	end
	return 0
end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, critical)
	if eventType ~= "SPELL_HEAL" and eventType ~= "SPELL_PERIODIC_HEAL" then
		return
	end
	if amount and amount > 0 then
		local isSmember = bit.band(srcFlags, LIB_FILTER_GROUP)
		isSmember = (isSmember == COMBATLOG_OBJECT_TYPE_PLAYER or isSmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isSmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isSmember then
			amount = amount - getOverheal(dstName, amount)
			
			self:CheckHPSTimer()

		 	local newTime = GetTime()
			combatStartTime[srcName] = combatStartTime[srcName] or newTime
			lastHealTime[srcName] = newTime
			healingDone[srcName] = ( healingDone[srcName] or 0 ) + amount	

			mod:UpdateHPS(srcName)
		end
	end
end

-----------------------------------------------------------------------
-- Combat status watching
-----------------------------------------------------------------------

function mod:PLAYER_REGEN_ENABLED()
	mod:ResetHPSTimer( playerName )
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] then
			if not UnitAffectingCombat( unit.unitid ) then
				isInCombat[name] = nil
				mod:ResetHPSTimer( name )
			else
				mod:CheckHPSTimer(name)			
			end
		elseif UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
