﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Healing")

L:RegisterTranslations("enUS", function() return {
	["Healing done"] = true,
	["H"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Healing done"] = "治疗输出",
	["H"] = "治疗输出(H)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Healing done"] = "造成治療",
	["H"] = "造成治療(H)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing done"] = "치유량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Healing done"] = "Soins prodigués",
	["H"] = "S",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local roster = AceLibrary("Roster-2.1")
local parserEvent = {
	eventType = "Heal",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Healing done"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName, "AceEvent-2.0")
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "healing"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Healing done"] end
function mod:GetShortDisplayName() return L["H"] end

local function getOverheal(unit, amount)
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit) - UnitHealth(unit)
		if d < amount then
			return amount - d
		end
	end
	return 0
end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, critical)
	if eventType ~= "SPELL_HEAL" and eventType ~= "SPELL_PERIODIC_HEAL" then
		return
	end
	if amount and amount > 0 then
		local isSmember = bit.band(srcFlags, LIB_FILTER_GROUP)
		isSmember = (isSmember == COMBATLOG_OBJECT_TYPE_PLAYER or isSmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isSmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isSmember then
			amount = amount - getOverheal(dstName, amount)
			self:OnData(srcName, amount)
		end
	end
end

