﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HealingTaken")

L:RegisterTranslations("enUS", function() return {
	["Healing taken"] = true,
	["HT"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Healing taken"] = "接受治疗",
	["HT"] = "接受治疗(HT)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Healing taken"] = "承受治療",
	["HT"] = "承受治療(HT)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing taken"] = "받은 치유량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Healing taken"] = "Soins reçus",
	["HT"] = "SR",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Healing taken"
local roster = AceLibrary("Roster-2.1")
local parserEvent = {
	eventType = "Heal",
	recipientID_not = false,	
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "healing"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Healing taken"] end
function mod:GetShortDisplayName() return L["HT"] end

local function getOverheal(unit, amount)
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			return amount - d
		end
	end
	return 0
end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, critical)
	if eventType ~= "SPELL_HEAL" and eventType ~= "SPELL_PERIODIC_HEAL" then
		return
	end
	if amount and amount > 0 then
		local isDmember = bit.band(dstFlags, LIB_FILTER_GROUP)
		isDmember = (isDmember == COMBATLOG_OBJECT_TYPE_PLAYER or isDmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isDmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isDmember then
			amount = amount - getOverheal(dstName, amount)
			self:OnData(dstName, amount)
		end
	end
end

